
module CMD
module Point_Gadget



def self.createCSV
	
	model = Sketchup.active_model
	ents = model.entities
	mrkrs = ents.grep(Sketchup::ComponentInstance).delete_if {|i| i.definition.name[0..8] != "pg_Marker"}
	
	
	expMkrs = Array.new
	pID = 0
	ptNOT = 0
	ptREADY = Array.new

	ptInfoArray = Array.new
	
	#builds export marker list of those not hidden or on a layer that is off.
	mrkrs.each {|i|
		if i.layer.visible?
			if i.visible?
				expMkrs << i
			end
		end
	}
	
	
	expMkrs.each {|i|
		if i.attribute_dictionary "ptGadget"
			ptREADY << i
		else
			ptNOT = ptNOT +1
		end
	}

	if ptNOT > 0
		message = "There are " + ptNOT.to_s + " markers that are not registered for export.\nWould you like to continue?"
		v = UI.messagebox message, MB_YESNO
		if v == 7; return false; end
	end
	
	prompt = ["Header","Order","Units"]
	default = ["YES","NEZ(yxz)","Feet"]
	list = ["YES|NO","NEZ(yxz)|ENZ(xyz)","Inches|Feet|Centimeters|Meters"]
	csvOp = UI.inputbox prompt, default, list, "CSV Export Options"
	
	if csvOp == false
		return 
	end
	
	
		
	#Save file dialog; create folder; create file
	suggested_name = File.basename(Sketchup.active_model.path).sub(/\.skp$/,".csv")
	@file_path = UI.savepanel("Save CSV File", "#{ENV['USERPROFILE']}/", suggested_name)
	return nil if @file_path.nil?
	@file_path = File.expand_path(@file_path).unpack("U*").map{|c|c.chr}.join.sub(/(?:\.\w{0,4})?$/, ".csv")
	@out = File.new(@file_path, "w")
	@out = File.open(@file_path, "w")
	
	#for user defined origin
	origin = Sketchup.active_model.get_attribute "ptGadget", "Origin_Point", Geom::Point3d.new(0,0,0)
	transform = Geom::Transformation.new(origin)

	ptREADY.each do |i|
		ptInfoRow = []
		if i.attribute_dictionary "ptGadget"
			ptDictionary = i.attribute_dictionary "ptGadget"
			keyList = ptDictionary.keys
			
			#generates ID automatically... to be removed to allow for static pointIDs
			#pID = pID + 1
			#ptInfoRow << pID.to_s + ","	
					
			keyList.each do |k|
			
				
				if k == "0_Pt_ID"; ptID = ptDictionary[k]
					ptInfoRow << ptID
				end
				
				if k == "1_Pt_Dynamic"
					ptAdjusted = ptDictionary[k].transform transform
					ptStr = ptAdjusted.to_a
					expCrd = Array.new
					ptStr.each {|v|
						
						if csvOp[2] == "Inches"; expCrd << sprintf("%.6f",v.to_inch);end
						if csvOp[2] == "Feet"; expCrd << sprintf("%.6f",v.to_feet);end
						if csvOp[2] == "Meters"; expCrd << sprintf("%.6f",v.to_m);end
						if csvOp[2] == "Centimeters"; expCrd << sprintf("%.6f",v.to_cm);end
						#if csvOp[2] == "Millimeters"; expCrd << sprintf("%.6f",v.to_mm);end
					}
					
					if csvOp[1] == "NEZ(yxz)"
						coord = []
						coord  << expCrd[1].to_s
						coord  << expCrd[0].to_s
						coord  << expCrd[2].to_s
						ptInfoRow << coord.join(",")
					end
					
					if csvOp[1] == "ENZ(xyz)"; ptInfoRow << expCrd.join(",");end
				end
				
				# Point Feature
				if k == "4_Pt_Feature"; ptFeature = ptDictionary[k].to_s
					ptInfoRow << ptFeature
				end
				
				
			end
			
			ptInfoArray << ptInfoRow
			
		end
		
	end
	
	#CSV HEADER ---------------------------------
	if csvOp[0] == "YES"
		units = csvOp[2].to_s
		if csvOp[1] == "NEZ(yxz)"; csvHdr = "PointID,N(Y),E(X),Z,Feature(Description),Units = " + units;end
		if csvOp[1] == "ENZ(xyz)"; csvHdr = "PointID,E(X),N(Y),Z,Feature(Description),Units = " + units;end
		@out.puts csvHdr
	end
		
	r = ptInfoArray.sort_by{|k|k[0] =~ /^p(\d+)q(\d+)$/ ; [$1.to_i, $2.to_i] }
	r.each { |x| 
		
		row = x[0].to_s + "," + x[1].to_s + "," + x[2].to_s + "\n"
		@out.puts row 
	}
	@out.close
	ptCNT = ptInfoArray.length.to_s
	message = "CSV export completed\n" + ptCNT + " Points"
	UI.messagebox message;
end

#end of modules
end
end