# Name :	Total Station Point Importer

module CMD
module Point_Gadget


class TSP_FileImport

	
	def self.create_dialog
	
	@htmlDlg = UI::WebDialog.new("Point Gadget: Point Importer File Options", false,600,600)
	@rbDirPath = File.dirname(File.expand_path(__FILE__))
	@rbDirPath.to_s
	html = @rbDirPath + "/tspt_importer.html"
	html = html.to_s
	@markers = []
	@removeIDs = false
	@htmlDlg.set_file(html)
	@htmlDlg.set_position(300,300)
	@htmlDlg.set_size(400,500)
	
		
		#Checks to see if any point markers already exist in model.  If YES, reassign IDs after import.
		@markers = Point_Gadget.findMarkerInstances(false)
		
		if @markers.length > 0
			option = UI.messagebox "** Warning **  ID numbers associated with points in the imported file will take priority over IDs numbers assigned to points already in this model.\n\nPre-existing points will be assigned new ID numbers.\n\nDo you wish to proceed?", MB_YESNO 
			if option == IDYES
				@removeIDs = true
			else
				@markers = []
				return
			end
		end
	
		@htmlDlg.add_action_callback("comeOnOver") { |d, a|
		
		fdel = @htmlDlg.get_element_value("fdel")
		funit = @htmlDlg.get_element_value("funit")
		crdOrder = @htmlDlg.get_element_value("fcrdord")
		fheader = @htmlDlg.get_element_value("fheader")
		
		@htmlDlg.close
		
		if @removeIDs == true
			@markers.each {|i|
				i.attribute_dictionary "ptGadget"
				i.set_attribute "ptGadget", "0_Pt_ID", "NOT SET"
			}
		end
		
		readPoints(fdel,funit,crdOrder,fheader)
		}
		
	@htmlDlg.show_modal
	
	end


	def self.readPoints(d,u,crdOrder,h)
		
		@rbDirPath = File.dirname(File.expand_path(__FILE__))
		@rbDirPath.to_s

		@suDataArray = Array.new
		@fileName
		
		@ptGroup = Array.new
 		@tgGroup = Array.new
 		@mkrGroup = Array.new
 
		g = Sketchup.active_model.active_entities

		chosen_pointfile = UI.openpanel "Select Total Station Point file (txt or csv)", nil, "*.txt;*.csv"

		# User cancels out of prompt
		if chosen_pointfile == nil
			return
		end
		
		@fileName = File.basename(chosen_pointfile)
		fileExt = File.extname(chosen_pointfile)
		fileExt = fileExt.downcase
		
		if fileExt != ".txt" && fileExt != ".csv"
			UI.messagebox("Please select a TXT or CSV file")
			return
		end

		lines = File.open(chosen_pointfile).readlines

		d.to_s
		u.to_s
		crdOrder.to_s
		h.to_s

		# -- If the first row is a column header, remove the first row from array
		if h == "YES"
			lines.shift
		end
		
		# -- Iterate through each line of the file and build an array for each line.
		lines.each do |s|
			

			case d
			when "comma"
			lineValues = s.split(',')
			when "tab"
			lineValues = s.split("\t")
			end
			
			pInfo = lineValues[0..4]
			pInfo.each {|a| a.strip! if a.respond_to? :strip! }
			
			# -- For each line, extract data needed for SketchUp [Pt ID, 3DPoint, Feature]
			suPointData = extractData(pInfo,u,crdOrder)

			if suPointData  == "-STOP-" || suPointData  == nil
				return
			end
				 
			next if suPointData  == "-GO-"
			
			@suDataArray.push(suPointData)
		
		end
		
		PointMarkerObserver.commitOpTrigger
		Sketchup.active_model.start_operation "ProcessSUData()"

		# --- @suDataArray ==> ptName,su3DPoint,ptLabel ----
		processSUData(@suDataArray)
		
		Sketchup.active_model.commit_operation          # End of UNDO capsule.
		PointMarkerObserver.commitOpTrigger


		
	end




# ---------------------------------------------------------------
# --- Extracts the data needed for SketchUp.  Returns an array 
# --- Returns an array consisting of 3 items:  PointID, Point3D Object, Point Feature
# ---------------------------------------------------------------

	def self.extractData(v,u,ord)

			
			ptName = v[0]
			
			a = v[1..3]
			
			chk = coordChk(a)
			if chk == false
				msg = "Coordnates value ERROR for Point ID:" + ptName + "\n Do you wish to skip and continue?"
				chkResult = UI.messagebox "Coordnates value ERROR for Point ID:" + ptName + "\n Do you wish to skip and continue?", MB_YESNO
				if chkResult == 6
					return "-GO-"
				else 
					return "-STOP-"
				end
			end
			
			
			if ord == "NEZ"
			# NEZ = YXZ
			  crdX = v[2]
			  crdY = v[1]
			else 
			# ENZ = XYZ
			  crdX = v[1]
			  crdY = v[2]
			end
			crdZ = v[3]
			
			if u == "Feet"
			  crdX = crdX.to_f.feet
			  crdY = crdY.to_f.feet
			  crdZ = crdZ.to_f.feet
			end
			
			if u == "Meters"
			  crdX = crdX.to_f.m
			  crdY = crdY.to_f.m
			  crdZ = crdZ.to_f.m
			end
			
			if u == "Inches"
			  crdX = crdX.to_f.inch
			  crdY = crdY.to_f.inch
			  crdZ = crdZ.to_f.inch
			end
			
			if u == "Centimeters"
			  crdX = crdX.to_f.cm
			  crdY = crdY.to_f.cm
			  crdZ = crdZ.to_f.cm
			end
			
			ptLabel = v[4]
			su3DPoint = get3DPoint(crdX,crdY,crdZ)
			data = [ptName,su3DPoint,ptLabel]
				
			return data
			 
		#end
						
	end




# ---------------------------------------------------------
# ---  Checks coordinates to make sure they are valid
# ---------------------------------------------------------

	def self.coordChk(crds)
		
		crds.each do |c|
			
			if c == "0" || c == ".0"
				c="0.0"
			end
			a = c.reverse
			a = a.sub(/^[0:]*/,"")
			a = a.reverse
			a = a.to_f
			a = a.to_s
			
			if a == "0."
				a="0.0"
			end
			
			b = a.to_f
			b = b.to_s
			if a != b 
				#UI.messagebox( a.to_s + " != " + b )
				return false
			end
			
		end
	
	end



	
# ---------------------------------------------------------
# ---  Create Points, Tags ... assign to group and layer
# ---  ptName,su3DPoint,ptTimestamp,ptLabel
# ---------------------------------------------------------

	def self.processSUData(pData)
		
		model = Sketchup.active_model
		ss = model.selection
		ss.clear
		
 		entities = model.active_entities

		
		# --- CREATE CONSTRUCTION POINT MARKERS ---		
		insertMarkers(pData)
		
		

		#Assign point markers to a group ----
=begin
		result = UI.messagebox "Do you want to place this import into a group?", MB_YESNO 
		if result ==IDYES
 			g= entities.add_group(@ptGroup)
 			g.name = @fileName + ": Imported Point Collection"
 		end
=end 		

		#Re-assigns IDs to pre-existing point markers.
		if (@markers)
			@markers.each {|i|
				id = Point_Gadget.getNewID
				i.attribute_dictionary "ptGadget"
				i.set_attribute "ptGadget", "0_Pt_ID", id
			}
			@markers = []
		end

	end



# --------------------------------------------
# --- Build array of all the points generated.
# --------------------------------------------

	def self.get3DPoint(x,y,z)
		point1 = Geom::Point3d.new(x,y,z)
		return point1
	end



# ---------------------------------------------
# --- Build construction points (id,point3d,feature)
# ---------------------------------------------
	
	def self.insertMarkers(p)
		
		model = Sketchup.active_model
		entities = model.active_entities
		sel = model.selection
 		layers = model.layers
 		
 		@rbDirPath = File.dirname(File.expand_path(__FILE__))
		@rbDirPath.to_s
		skpLoc = @rbDirPath + "/pg_Marker.skp"
		marker = model.definitions.load skpLoc
		
		

 		p.each { |d|
			
			# Create the layer if it doesnt already exist ----
			f = d[2].to_s
			lyrF = d[2].to_s + " (Label)"
			if layers[lyrF] == nil
 				layers.add lyrF
			end
			
			id = d[0]
			loc = d[1]
			
			pt0 = Geom::Point3d.new(loc)
			origin = Sketchup.active_model.get_attribute "ptGadget", "Origin_Point", Geom::Point3d.new(0,0,0)
			vec = Geom::Vector3d.new(origin.x,origin.y,origin.z)
			pt1 = pt0 - vec
			xyz1 = pt1.to_a
			
			cpLoc = Geom::Transformation.new xyz1
			i = model.entities.add_instance marker, cpLoc
			i.layer = lyrF
			i.name = f
			i.material = "DimGray"
			Point_Gadget.setDynamic3dPoint(i,id,loc,f,false)
			sel.clear
			sel.add i
			#Refresh inserted marker to set all properties - needed incase of custom origin
			Point_Gadget.instanceUpdate
			sel.clear
			@ptGroup.push(i)
			
		}	
		
		model.commit_operation

	end	

end


#end of modules
end
end