# Name :	Section Finder


require 'sketchup.rb'

module CMD
module Point_Gadget

class ReportPage

@markers = []

	def initiatePointReport
	
		ss= Sketchup.active_model.selection
		if ss.length > 0
			@markers = ss.grep(Sketchup::ComponentInstance).delete_if {|i| i.definition.name[0..8] != "pg_Marker"}
			@markers.delete_if {|i| !(i.attribute_dictionary "ptGadget")}
			gents = ss.grep(Sketchup::Group)
			gents.each {|g|
				i = g.entities.grep(Sketchup::ComponentInstance).delete_if {|i| i.definition.name[0..8] != "pg_Marker"}
				@markers << i
				}
			@markers.flatten!
			
			if @markers.length == 0
				UI.messagebox "No point markers selected"
				return
			else
				create_dialog
			end
		end
		
		if ss.length == 0
			
			@markers = Point_Gadget.findMarkerInstances(false)
			
			if @markers.length == 0
				UI.messagebox "There are no point markers to create a report."
				return
			else
				create_dialog
			end
			
		end
		
		
	
	end


	def create_dialog

		@pg_dialog = UI::WebDialog.new("Point Report", true,600,600)
		rbDirPath = File.dirname(File.expand_path(__FILE__))
		rbDirPath.to_s
		html = rbDirPath + "/report_page.html"
		html = html.to_s	
		@pg_dialog.set_file(html)
		#@pg_dialog.set_position(200,200)
		@pg_dialog.set_size(575,410)


		# -------------------------------------------------------
		# Builds the dialog page from the dialogs initial loading
		# -------------------------------------------------------
		@pg_dialog.add_action_callback("htmlPageBld") { |d, a|
			pointData = getPointInfo
			js_command = "getPoints('" + pointData +"')"
			@pg_dialog.execute_script(js_command)
			#js_command2 = "sortGo()"
			#@pg_dialog.execute_script(js_command2)
		}

		
		
		# -------------------------------------------------------
		# Selects the points from dialog
		# -------------------------------------------------------
		@pg_dialog.add_action_callback("selectPoint") { |d, a|
			pointFocus(a)
		}
		
		if RUBY_PLATFORM.include?('darwin')
			@pg_dialog.show_modal
		else
			@pg_dialog.show
		end
		@pg_dialog.bring_to_front

		
		# Initiates the table sorting
		
		
	end

	def pointFocus(i)
	
		sID = i.to_s
	
		selection = Sketchup.active_model.selection
		selection.clear
		
		@markers.each do|e|
		 if e.valid?
		  val = e.entityID.to_s
		  if val == sID
			selection.add e
		  end
		 end
		end
	end

# -------------------------------------------------------
# Generates a string with all point information
# to be used by the web dialog's javascript
# -------------------------------------------------------


	def getPointInfo
		ptInfoArray = []
			
		#for user defined origin
		origin = Sketchup.active_model.get_attribute "ptGadget", "Origin_Point", Geom::Point3d.new(0,0,0)
		transform = Geom::Transformation.new(origin)
				
		if (@markers.length > 0)
			unit = ""
			unitOptions = Sketchup.active_model.options["UnitsOptions"]
			unitOptions.each {|k,v| if k == "LengthUnit"; unit = v; break; end}
			
			pt_o = origin.to_a
			
			#pt_o = origin.to_a.join(",") + "-brk-"
			
			
			if unit == 0; pt_o.map!{|v| sprintf("%.6f",v.to_inch)}; end
			if unit == 1; pt_o.map!{|v| sprintf("%.6f",v.to_feet)}; end
			if unit == 2; pt_o.map!{|v| sprintf("%.6f",v.to_mm)}; end
			if unit == 3; pt_o.map!{|v| sprintf("%.6f",v.to_cm)}; end
			if unit == 4; pt_o.map!{|v| csprintf("%.6f",v.to_m)}; end
			
			
			@markers.each {|i|
				ptInfoRow = []
				if i.attribute_dictionary "ptGadget"
					ptDictionary = i.attribute_dictionary "ptGadget"
					keyList = ptDictionary.keys
					
					#HTML Column 1 - Marker Entity ID					
					ptInfoRow << i.entityID
					
					keyList.each {|k|
			
						#HTML Column 2 - Point ID
						if k == "0_Pt_ID"; ptID = ptDictionary[k]
							ptInfoRow << ptID
						end
						
						#HTML Column 3,4,5 - XYZ coordinates
						if k == "1_Pt_Dynamic"
							ptAdjusted = ptDictionary[k].transform transform
							coords = ptAdjusted.to_a
							crds = []
							if unit == 0; coords.map!{|v| crds << sprintf("%.6f",v.to_inch)}; end
							if unit == 1; coords.each {|v| crds << sprintf("%.6f",v.to_feet)}; end
							if unit == 2; coords.each {|v| crds << sprintf("%.6f",v.to_mm)}; end
							if unit == 3; coords.each {|v| crds << sprintf("%.6f",v.to_cm)}; end
							if unit == 4; coords.each {|v| crds << sprintf("%.6f",v.to_m)}; end			
							ptInfoRow << crds.join(",")
						end
				
						#HTML Column 6 - Point feature information
						if k == "4_Pt_Feature"
							ptFeature = ptDictionary[k].to_s
							ptInfoRow << ptFeature
						end
					}
			
					ptInfoArray << ptInfoRow
				end
			}
		end
		
		if (@markers.length > 0)
			pointData = ""
			#Sort multi-dimensional array by Point ID
			r = ptInfoArray.sort_by{|k|k[1] =~ /^p(\d+)q(\d+)$/ ; [$1.to_i, $2.to_i] }
		
			r.each { |x| 
				id = x[1]
				row = x[0].to_s + "-col-" + id.to_s + "-col-" + x[2].to_s + "-col-" + x[3].to_s + "-brk-"
				pointData << row
			}
			
			pointOrigin = pt_o.join(",") + "-brk-"
			pointData.insert(0, pointOrigin)
			
			pointData.chomp!("-brk-")
			
			return pointData
		end
	end
	
		

# End of Class
end 

#end of modules
end
end