
module CMD
module Point_Gadget

@pnt = Geom::Point3d.new(0,0,0)

class PW_Shoot



	def initialize
		@@ip1 = nil
		@ip2 = nil
		@xdown = 0
		@ydown = 0	
	end

	def activate
		model=Sketchup.active_model
		if Sketchup.version.to_i<7
			model.start_operation "Draw points"
		else
			model.start_operation "Draw points",true
		end
		
		prompt = ["Feature:"]
		default = ["Point"]
		@fInfo = UI.inputbox prompt, default, "Point Feature Description"
		#if false (canceled), exit the tool and default to the select tool
		if @fInfo == false ; Sketchup.send_action "selectSelectionTool:"; return; end
				
		@ip = Sketchup::InputPoint.new
		@@ip1 = Sketchup::InputPoint.new
		@ip2 = Sketchup::InputPoint.new
		@drawn=false
		
		if Sketchup.active_model.rendering_options["HideConstructionGeometry"]==true 
			Sketchup.active_model.rendering_options["HideConstructionGeometry"]=false 
		end
		self.reset(nil)
	end
  
	def deactivate(view)
		view.invalidate if @drawn;
		#puts "Shooter Tool has been deactivated in view: " + view.to_s
	end
	
	def onCancel(flag, view);self.reset(view);end
  
	def reset(view)
				
		@state = 0
		Sketchup::set_status_text("Specify first Point:")
		@ip.clear
		@@ip1.clear
		@ip2.clear
		if view
			view.tooltip=nil
			view.invalidate if @drawn
		end
		@drawn = false
	end
	
	def onMouseMove(flags, x, y, view)
		if @state==0
			@ip.pick view, x, y
			if @ip != @@ip1
				view.invalidate if @ip.display? or @@ip1.display?
				@@ip1.copy! @ip
				view.tooltip = @@ip1.tooltip
			end
			view.tooltip = "First point:"
		else
			@ip2.pick view,x,y,@@ip1
			view.tooltip=@ip2.tooltip if @ip2.valid?
			view.invalidate
			if @ip2.valid?
				length=@@ip1.position.distance(@ip2.position)
				Sketchup::set_status_text(length.to_s, SB_VCB_VALUE)
			end
			view.tooltip="Next point:"
		end
	end

	def onLButtonDown(flags, x, y, view)
		if @state==0
			@@ip1.pick view, x, y
			if @@ip1.valid?
				@state = 1
				Sketchup::set_status_text("Click for next shot:")
				Sketchup::set_status_text("Length", SB_VCB_LABEL)
				@xdown = x
				@ydown = y
			end
		elsif @ip2.valid? && @ip2!=@@ip1
			@state = 1
			@@ip1.copy! @ip2	
		end
		view.lock_inference
	end

	def onLButtonUp(flags, x, y, view)
		if @@ip1.valid?
			self.place_marker(@@ip1.position,view) if self.no_points_here?(@@ip1.position)
		elsif @ip2.valid?
			self.place_marker(@ip2.position,view) if self.no_points_here?(@ip2.position)
		end
	end

	def onKeyDown(key, repeat, flags, view)
		if key == CONSTRAIN_MODIFIER_KEY && repeat == 1
			@shift_down_time = Time.now
			if( view.inference_locked? )
				view.lock_inference
			elsif @state==0 && @@ip1.valid?
				view.lock_inference @@ip1
				# view.line_width = 3
			elsif @state==1 && @ip2.valid?
				view.lock_inference @ip2, @@ip1
				# view.line_width = 3
			end
		end
	end

	def onKeyUp(key, repeat, flags, view)
		view.lock_inference if key==CONSTRAIN_MODIFIER_KEY && view.inference_locked? && (Time.now - @shift_down_time)>0.5
	end

	def onUserText(text, view)
		return if @state == 0
		return if not @ip2.valid?
		begin
			value = text.to_l
		rescue
			#puts "Cannot convert #{text} to a Length"
			value = nil
			Sketchup::set_status_text("", SB_VCB_VALUE)
		end
		return if !value
		pt1 = @@ip1.position
		vec = @ip2.position - pt1
		return if vec.length == 0.0
		vec.length = value
		pt2 = pt1 + vec
		self.place_marker(pt2, view) if self.no_points_here?(pt2)
		@state = 1
		pt2 = view.screen_coords pt2
		@@ip1.pick view,pt2[0],pt2[1]
	end
  
	def draw(view)
		
	    if @@ip1.valid?
			if @@ip1.display?
				@@ip1.draw(view)
				@drawn = true
			end
			if @ip2.valid?
				@ip2.draw(view) if @ip2.display?
				view.set_color_from_line(@@ip1, @ip2)
				self.draw_geometry(@@ip1.position, @ip2.position, view)
				@drawn = true
			end
		end
	end


#### Edited - places the point marker
	def place_marker(p,view)
		model=Sketchup.active_model
		model.start_operation "placeMarker"
		
		@rbDirPath = File.dirname(File.expand_path(__FILE__))
		@rbDirPath.to_s
		skpLoc = @rbDirPath + "/pg_Marker.skp"
		marker = model.definitions.load skpLoc
		
		id = Point_Gadget.getNewID
		
		pntLoc = Geom::Transformation.new p
 		i = view.model.entities.add_instance marker, pntLoc
 		i.name = @fInfo[0].to_s
 		f = i.name
 		
 		# inserts a Point3d in attribute library
 		Point_Gadget.setDynamic3dPoint(i,id,p,f,true)
 		#commit_operation should be placed before the addition of the observer.
 		
 		model.commit_operation
 		i.add_observer(PointMarkerObserver.new)
		view.invalidate
		
	end

	def draw_geometry(pt1, pt2, view)
		view.draw_line(pt1, pt2)
		view.invalidate
	end
	
	def no_points_here?(p)
		Sketchup.active_model.entities.select{|e|e.class==Sketchup::ConstructionPoint}.each do |e|
			return false if e.position.to_a==p.to_a
		end
		return true
	end

#end of tool		
end


#end of modules
end
end

