
=begin

Author:     	Chris Dizon
				Additional Credits:  Matt666
Name:      		Point Gadget
Version:   		1.0

				1.000   08/26/2013
				
=end


require 'sketchup.rb'

module CMD
module Point_Gadget

@mrkrIntCpointLoc = nil
@markerName = "pg_Marker"
#Internal location of the component's imbedded construction point.
#This location is unique to the marker component "pg_Marker"
@pnt = Geom::Point3d.new(0,0,0)



class PointMarkerObserver < Sketchup::EntityObserver
	@@commitTrig = false
	
	def self.commitOpTrigger
		if @@commitTrig == false
		  @@commitTrig = true
		else
		  @@commitTrig = false
		end
	end 

	def onChangeEntity(entity)
		id = entity.entityID
		if (id > 0) && (@@commitTrig == false)
			Point_Gadget.instanceUpdate
		end
	end
end


def self.setOrigin

	model = Sketchup.active_model
	model.start_operation "imputXYZ"
	
	origin = Sketchup.active_model.get_attribute "ptGadget", "Origin_Point", Geom::Point3d.new(0,0,0)
	menuText = "Current Origin = " + origin.to_s
	
	prompts = ["Units","X","Y","Z"]
	default = ["Ft",0.0,0.0,0.0]
	list = ["in|Ft|cm|M","","",""]
	begin
	vals = UI.inputbox prompts, default, list,  menuText
	rescue
		UI.messagebox("One or more of your point values are invalid numbers.\nPlease try again.")
		vals = false
	end
	
	if vals!= false || nil
		units = vals[0]
		xyz = vals[1..3]
		
		if units == "in"; xyz.map!{|c|  c.inch};end
		if units == "Ft"; xyz.map!{|c| c.feet};end
		if units == "cm"; xyz.map!{|c| c.cm};end
		if units == "M"; xyz.map!{|c| c.m};end
		
		pt_origin = Geom::Point3d.new(xyz[0],xyz[1],xyz[2])
		
		model = Sketchup.active_model
		model.attribute_dictionary "ptGadget", true
		model.set_attribute "ptGadget", "Origin_Point", Geom::Point3d.new(xyz[0],xyz[1],xyz[2])
	end
	
end



def self.buildPointList(x)
	idAry << x
	model = Sketchup.active_model
	ptDictionary = model.attribute_dictionary "ptGadget", true
	ptDictionary.set_attribute "pwGadget", "ID_LIST", x
end

# Finds all point markers including markers that reside within a group.
# TRUE = Returns an array of ALL markers  FALSE = Returns only registered markers
def self.findMarkerInstances(all)
	ents = Sketchup.active_model.entities
	markers = ents.grep(Sketchup::ComponentInstance).delete_if {|i| i.definition.name[0..8] != "pg_Marker"}
	
	gents = ents.grep(Sketchup::Group)
	gents.each {|g|
		i = g.entities.grep(Sketchup::ComponentInstance).delete_if {|i| i.definition.name[0..8] != "pg_Marker"}
		markers << i
	}
	markers.flatten!
	if all == false
		mTrim = Array.new
		markers.each {|i|
		attrdict = i.attribute_dictionary "ptGadget"
		if (attrdict); mTrim<<i; end
		}
		markers.clear
		markers = mTrim
	end
	return markers
end

def  self.getNewID
	idCheck = []
	markers = findMarkerInstances(true)
	markers.each {|i|
		attrdict = i.attribute_dictionary "ptGadget"
		
		if (attrdict)
			pid = attrdict["0_Pt_ID"]
			if (pid) && (pid != "NOT SET"); idCheck << pid.to_i; end
		end
	}

	idCheck.sort!
	topID = idCheck.last
	newID = topID.to_i + 1
	#newID = n.floor.to_s
	idCheck.clear
	return newID
end

##################################################
## Creates and populates dictionary attributes
##################################################

def self.setDynamic3dPoint(i,id,p,f,l)
		
		lib = i.attribute_dictionary "ptGadget", true
		i.set_attribute "ptGadget", "0_Pt_ID", id
		i.set_attribute "ptGadget", "1_Pt_Dynamic", p
		i.set_attribute "ptGadget", "4_Pt_Feature", f
		i.set_attribute "ptGadget", "Pt Static", p.to_a
		i.set_attribute "ptGadget", "Pt_DescFeature", f
	
		i.material = "Lime"
		if l == true
			model = Sketchup.active_model
			layers = model.layers
			fLayer = f.to_s + " (Label)"
			if layers[fLayer] == nil
				layers.add fLayer
			end
			i.layer = fLayer
		end	
end




####################################################################
## Updates all instances: Prepairs each for export/reporting
####################################################################

def self.instanceUpdate
	view = Sketchup.active_model.active_view	
	ss = Sketchup.active_model.selection
	
	if ss.length==0
		return UI.messagebox('Please make a selection.')
		#ents = Sketchup.active_model.entities
		#markers = Sketchup.active_model.entities.grep(Sketchup::ComponentInstance).delete_if {|i| i.definition.name[0..8] != "pg_Marker"}
		#markers.each {|i| ss.add i}
		
	else
		selSet = ss.to_a # Creating an array from the ss so not to modify the the collection it is iterating through (bad for 2014).
		selSet.each do |c|
			if c.is_a?(Sketchup::ComponentInstance) && c.definition.name[0..8] == "pg_Marker"
				ss.add c
			else
				ss.remove c
			end
		end
	end


	ss.each do |i|
		if i.is_a?(Sketchup::ComponentInstance) && i.definition.name[0..8] == "pg_Marker"
			t = i.transformation
			@pnt = Geom::Point3d.new(0,0,0).transform! t
			f = i.name
			
			iDict = i.attribute_dictionary "ptGadget"
			
			if (iDict)
				id = iDict["0_Pt_ID"]
				if id == "NOT SET"; id = getNewID;end
				setDynamic3dPoint(i,id,@pnt,f,true)
			else
				id = getNewID
				setDynamic3dPoint(i,id,@pnt,f,true)
			end
			i.add_observer(PointMarkerObserver.new)
		end
		
	end
	view.invalidate
	#puts ss.length.to_s + " points have been updated"
end


####################################################################
## Converts construction points to markters
####################################################################

def self.convertCP
	view = Sketchup.active_model.active_view	
	model = Sketchup.active_model
	entities = model.entities
	
	cpA =[]
	ss = Sketchup.active_model.selection
	
	model.start_operation "convertCp"
	
	#If not selection made, get all construction points in the model
	# and insert the construction points into the selection set
	if ss.length==0

		entities = Sketchup.active_model.entities
		entities.each do |c|
			if c.is_a? Sketchup::ConstructionPoint ; cpA << c ; ss.add c; end
		end
	#If selection made, get all construction points in selection
	# and remove other entities from selection set.
	else 
		ss.each do |c|
			if c.is_a? Sketchup::ConstructionPoint
				cpA << c
			else
				ss.remove c
			end
		end
	end
	
	
	len = cpA.length
	if len > 0
		q = len.to_s + " construction points will be converted.\nDo you wish to continue?"
		updateYN = UI.messagebox q, MB_YESNO
		if updateYN == 6
			
			prompt = ["Feature:"]
			default = ["Point"]
			fInfo = UI.inputbox prompt, default, "Point Feature Description"
			if fInfo == false ; Sketchup.send_action "selectSelectionTool:"; return; end
				
			@rbDirPath = File.dirname(File.expand_path(__FILE__))
			@rbDirPath.to_s
			skpLoc = @rbDirPath + "/pg_Marker.skp"
			marker = model.definitions.load skpLoc
			
			cpA.each do |s|
				#lname = s.layer
				pos = s.position
				cpLoc = Geom::Transformation.new pos
				i = model.entities.add_instance marker, cpLoc
				#i.layer = lname
				i.name = fInfo.to_s
				ss.add i
				ss.remove s
				
				
		
			end
			
			
			
		else
		 return;
		end
	else
		return UI.messagebox("There are no construction points to convert")
	end
	
	entities.erase_entities cpA
	instanceUpdate
	view.invalidate
	
	model.commit_operation
	ss.clear
	
end


####################################################################
## Input XYZ feature
####################################################################

def self.inputXYZ
	model = Sketchup.active_model
	model.start_operation "imputXYZ"
	
	prompts = ["Name","Units","X","Y","Z"]
	default = ["Point","Ft",0.0,0.0,0.0]
	list = ["","in|Ft|cm|M","","",""]
	begin
	vals = UI.inputbox prompts, default, list,  "Input point coordinates"
	rescue
		UI.messagebox("One or more of your point values are invalid numbers.\nPlease try again.")
		vals = false
	end
	
	if vals!= false || nil
		units = vals[1]
		xyz = vals[2..4]
		name = vals[0].to_s
		
=begin		
o = Geom::Point3d.new(120,120,120)
p = Geom::Point3d.new(360,360,360)

vec = Geom::Vector3d.new(o.x,o.y,o.z)
npt = p - vec

transform = Geom::Transformation.new(o)
a = npt.transform transform
puts a
=end
		if units == "in"; xyz.map!{|c|  c.inch};end
		if units == "Ft"; xyz.map!{|c| c.feet};end
		if units == "cm"; xyz.map!{|c| c.cm};end
		if units == "M"; xyz.map!{|c| c.m};end
		
		pt0 = Geom::Point3d.new(xyz)
		origin = Sketchup.active_model.get_attribute "ptGadget", "Origin_Point", Geom::Point3d.new(0,0,0)
		vec = Geom::Vector3d.new(origin.x,origin.y,origin.z)
		pt1 = pt0 - vec
		xyz1 = pt1.to_a
		
		@rbDirPath = File.dirname(File.expand_path(__FILE__))
		@rbDirPath.to_s
		skpLoc = @rbDirPath + "/pg_Marker.skp"
		
		marker = model.definitions.load skpLoc
		 
		trans = Geom::Transformation.new xyz1
		i = model.entities.add_instance marker, trans
		i.name = name
		
		id = "NOT SET"
		setDynamic3dPoint(i,id,xyz,name,true)
		
		ss = model.selection
		ss.clear
		ss.add i
		instanceUpdate

		model.commit_operation
		
		i.add_observer(PointMarkerObserver.new)
		
 	end
 	return
 	
end


####################################################################
## Create LineWork
####################################################################
def self.createLinework

	model = Sketchup.active_model
	ss = model.selection
	len = ss.length
	siblingsAry = []
	idAry = []
	lineworkPoints = []
	lTag = "false"
	
	if len != 1; return UI.messagebox('Make only 1 selection');
	else 
		if ss[0].is_a?(Sketchup::ComponentInstance) && ss[0].definition.name[0..8] == "pg_Marker"
			s_dictionary = ss[0].attribute_dictionary "ptGadget"
			if s_dictionary != nil
				lTag = s_dictionary["4_Pt_Feature"]

				mrks = findMarkerInstances(false)

				#finds all markers with same label (siblings)
				mrks.each {|m|
					dict = m.attribute_dictionary "ptGadget"
					mTag = dict["4_Pt_Feature"]
					if lTag == mTag
						siblingsAry << m
						mID = dict["0_Pt_ID"].to_i
						idAry << mID

					end
				}
				
				idAry.sort!
				idAry.each {|i|
					siblingsAry.each {|m|
						dict = m.attribute_dictionary "ptGadget"
						mTag = dict["0_Pt_ID"].to_i
						if i == mTag; lineworkPoints << dict["1_Pt_Dynamic"]; end
					}	
				}
					
			else 
				return UI.messagebox('This point marker has not been registered with the Point Gadget plugin. Please use the Update tool to register the marker.')
			end
		else
			return UI.messagebox('Please select a Point Gadget marker.')
		end
	end
	
	len = lineworkPoints.length
	if lineworkPoints.length > 1
		
		ents = model.entities
		group = ents.add_group
		lgroup = group.entities
		lgroup.add_edges lineworkPoints
		gName = lTag + " (Linework)"
		group.name = gName
		layers = model.layers
		
		#if layer does not exist, create it
		if layers[gName] == nil
 			layers.add gName
		end
		# assign group to layer
		group.layer = gName
		#lw = Sketchup.active_model.entities.add_edges lineworkPoints
		
	
	else 
		return UI.messagebox('There are not enough similar markers to create linework')
	end
	
	
	
end



####################################################################
## Color by layer
####################################################################

def self.colorByLayer
	model = Sketchup.active_model
	cbl = model.rendering_options["DisplayColorByLayer"]
	if cbl == false
		model.rendering_options["DisplayColorByLayer"]= true
	else
		model.rendering_options["DisplayColorByLayer"]= false
	end
end


#end of modules
end
end